@@ -1,6 +1,7 @@
 /***************************************************************************
  *   Copyright (C) 2004-2008 by Albert Astals Cid <tsdgeos@terra.es>       *
  *   Copyright (C) 2004 by Enrico Ros <eros.kde@email.it>                  *
+ *   Copyright (C) 2012 by Guillermo A. Amaral B. <gamaral@kde.org>        *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -45,14 +46,22 @@
 
 #include <config-okular-poppler.h>
 
+#include <algorithm>
+
+#ifdef HAVE_POPPLER_0_20
+#  include <poppler-media.h>
+#endif
+
+#include "annots.h"
 #include "formfields.h"
 #include "popplerembeddedfile.h"
 
-#ifdef HAVE_POPPLER_0_9
+Q_DECLARE_METATYPE(Poppler::Annotation*)
 Q_DECLARE_METATYPE(Poppler::FontInfo)
+#ifdef HAVE_POPPLER_0_20
+Q_DECLARE_METATYPE(const Poppler::LinkMovie*)
 #endif
 
-static const int PDFDebug = 4710;
 static const int defaultPageWidth = 595;
 static const int defaultPageHeight = 842;
 
@@ -63,11 +72,30 @@ class PDFOptionsPage : public QWidget
        {
            setWindowTitle( i18n( "PDF Options" ) );
            QVBoxLayout *layout = new QVBoxLayout(this);
+           m_printAnnots = new QCheckBox(i18n("Print annotations"), this);
+           m_printAnnots->setToolTip(i18n("Include annotations in the printed document"));
+           m_printAnnots->setWhatsThis(i18n("Includes annotations in the printed document. You can disable this if you want to print the original unannotated document."));
+           layout->addWidget(m_printAnnots);
            m_forceRaster = new QCheckBox(i18n("Force rasterization"), this);
            m_forceRaster->setToolTip(i18n("Rasterize into an image before printing"));
            m_forceRaster->setWhatsThis(i18n("Forces the rasterization of each page into an image before printing it. This usually gives somewhat worse results, but is useful when printing documents that appear to print incorrectly."));
            layout->addWidget(m_forceRaster);
            layout->addStretch(1);
+
+#ifndef HAVE_POPPLER_0_20
+           m_printAnnots->setVisible( false );
+#endif
+           setPrintAnnots( true ); // Default value
+       }
+
+       bool printAnnots()
+       {
+           return m_printAnnots->isChecked();
+       }
+
+       void setPrintAnnots( bool printAnnots )
+       {
+           m_printAnnots->setChecked( printAnnots );
        }
 
        bool printForceRaster()
@@ -81,6 +109,7 @@ class PDFOptionsPage : public QWidget
        }
 
     private:
+        QCheckBox *m_printAnnots;
         QCheckBox *m_forceRaster;
 };
 
@@ -143,7 +172,6 @@ Okular::Sound* createSoundFromPopplerSound( const Poppler::SoundObject *popplerS
     return sound;
 }
 
-#ifdef HAVE_POPPLER_0_9
 Okular::Movie* createMovieFromPopplerMovie( const Poppler::MovieObject *popplerMovie )
 {
     Okular::Movie *movie = new Okular::Movie( popplerMovie->url() );
@@ -151,112 +179,164 @@ Okular::Movie* createMovieFromPopplerMovie( const Poppler::MovieObject *popplerM
     movie->setRotation( (Okular::Rotation)( popplerMovie->rotation() / 90 ) );
     movie->setShowControls( popplerMovie->showControls() );
     movie->setPlayMode( (Okular::Movie::PlayMode)popplerMovie->playMode() );
+    movie->setAutoPlay( false ); // will be triggered by external MovieAnnotation
+    return movie;
+}
+
+#ifdef HAVE_POPPLER_0_20
+Okular::Movie* createMovieFromPopplerScreen( const Poppler::LinkRendition *popplerScreen )
+{
+    Poppler::MediaRendition *rendition = popplerScreen->rendition();
+    Okular::Movie *movie = 0;
+    if ( rendition->isEmbedded() )
+        movie = new Okular::Movie( rendition->fileName(), rendition->data() );
+    else
+        movie = new Okular::Movie( rendition->fileName() );
+    movie->setSize( rendition->size() );
+    movie->setShowControls( rendition->showControls() );
+    movie->setPlayMode( Okular::Movie::PlayOnce );
+    movie->setAutoPlay( rendition->autoPlay() );
     return movie;
 }
 #endif
 
+/**
+ * Note: the function will take ownership of the popplerLink object.
+ */
 Okular::Action* createLinkFromPopplerLink(const Poppler::Link *popplerLink)
 {
-	Okular::Action *link = 0;
-	const Poppler::LinkGoto *popplerLinkGoto;
-	const Poppler::LinkExecute *popplerLinkExecute;
-	const Poppler::LinkBrowse *popplerLinkBrowse;
-	const Poppler::LinkAction *popplerLinkAction;
-	const Poppler::LinkSound *popplerLinkSound;
-#ifdef HAVE_POPPLER_0_9
-	const Poppler::LinkJavaScript *popplerLinkJS;
+    Okular::Action *link = 0;
+    const Poppler::LinkGoto *popplerLinkGoto;
+    const Poppler::LinkExecute *popplerLinkExecute;
+    const Poppler::LinkBrowse *popplerLinkBrowse;
+    const Poppler::LinkAction *popplerLinkAction;
+    const Poppler::LinkSound *popplerLinkSound;
+    const Poppler::LinkJavaScript *popplerLinkJS;
+#ifdef HAVE_POPPLER_0_20
+    const Poppler::LinkMovie *popplerLinkMovie;
 #endif
-	Okular::DocumentViewport viewport;
-	
-	switch(popplerLink->linkType())
-	{
-		case Poppler::Link::None:
-		break;
-	
-		case Poppler::Link::Goto:
-		{
-			popplerLinkGoto = static_cast<const Poppler::LinkGoto *>(popplerLink);
-#ifdef HAVE_POPPLER_0_11
-			const Poppler::LinkDestination dest = popplerLinkGoto->destination();
-			const QString destName = dest.destinationName();
-			if (destName.isEmpty())
-			{
-				fillViewportFromLinkDestination( viewport, dest );
-				link = new Okular::GotoAction(popplerLinkGoto->fileName(), viewport);
-			}
-			else
-			{
-				link = new Okular::GotoAction(popplerLinkGoto->fileName(), destName);
-			}
-#else
-			fillViewportFromLinkDestination( viewport, popplerLinkGoto->destination() );
-			link = new Okular::GotoAction(popplerLinkGoto->fileName(), viewport);
+    Okular::DocumentViewport viewport;
+
+    bool deletePopplerLink = true;
+
+    switch(popplerLink->linkType())
+    {
+        case Poppler::Link::None:
+        break;
+
+        case Poppler::Link::Goto:
+        {
+            popplerLinkGoto = static_cast<const Poppler::LinkGoto *>(popplerLink);
+            const Poppler::LinkDestination dest = popplerLinkGoto->destination();
+            const QString destName = dest.destinationName();
+            if (destName.isEmpty())
+            {
+                fillViewportFromLinkDestination( viewport, dest );
+                link = new Okular::GotoAction(popplerLinkGoto->fileName(), viewport);
+            }
+            else
+            {
+                link = new Okular::GotoAction(popplerLinkGoto->fileName(), destName);
+            }
+        }
+        break;
+
+        case Poppler::Link::Execute:
+            popplerLinkExecute = static_cast<const Poppler::LinkExecute *>(popplerLink);
+            link = new Okular::ExecuteAction( popplerLinkExecute->fileName(), popplerLinkExecute->parameters() );
+        break;
+
+        case Poppler::Link::Browse:
+            popplerLinkBrowse = static_cast<const Poppler::LinkBrowse *>(popplerLink);
+            link = new Okular::BrowseAction( popplerLinkBrowse->url() );
+        break;
+
+        case Poppler::Link::Action:
+            popplerLinkAction = static_cast<const Poppler::LinkAction *>(popplerLink);
+            link = new Okular::DocumentAction( (Okular::DocumentAction::DocumentActionType)popplerLinkAction->actionType() );
+        break;
+
+        case Poppler::Link::Sound:
+        {
+            popplerLinkSound = static_cast<const Poppler::LinkSound *>(popplerLink);
+            Poppler::SoundObject *popplerSound = popplerLinkSound->sound();
+            Okular::Sound *sound = createSoundFromPopplerSound( popplerSound );
+            link = new Okular::SoundAction( popplerLinkSound->volume(), popplerLinkSound->synchronous(), popplerLinkSound->repeat(), popplerLinkSound->mix(), sound );
+        }
+        break;
+
+        case Poppler::Link::JavaScript:
+        {
+            popplerLinkJS = static_cast<const Poppler::LinkJavaScript *>(popplerLink);
+            link = new Okular::ScriptAction( Okular::JavaScript, popplerLinkJS->script() );
+        }
+        break;
+
+#ifdef HAVE_POPPLER_0_20
+        case Poppler::Link::Rendition:
+            // not implemented
+        break;
 #endif
-		}
-		break;
-		
-		case Poppler::Link::Execute:
-			popplerLinkExecute = static_cast<const Poppler::LinkExecute *>(popplerLink);
-			link = new Okular::ExecuteAction( popplerLinkExecute->fileName(), popplerLinkExecute->parameters() );
-		break;
-		
-		case Poppler::Link::Browse:
-			popplerLinkBrowse = static_cast<const Poppler::LinkBrowse *>(popplerLink);
-			link = new Okular::BrowseAction( popplerLinkBrowse->url() );
-		break;
-		
-		case Poppler::Link::Action:
-			popplerLinkAction = static_cast<const Poppler::LinkAction *>(popplerLink);
-			link = new Okular::DocumentAction( (Okular::DocumentAction::DocumentActionType)popplerLinkAction->actionType() );
-		break;
-		
-		case Poppler::Link::Sound:
-		{
-			popplerLinkSound = static_cast<const Poppler::LinkSound *>(popplerLink);
-			Poppler::SoundObject *popplerSound = popplerLinkSound->sound();
-			Okular::Sound *sound = createSoundFromPopplerSound( popplerSound );
-			link = new Okular::SoundAction( popplerLinkSound->volume(), popplerLinkSound->synchronous(), popplerLinkSound->repeat(), popplerLinkSound->mix(), sound );
-		}
-		break;
-		
-#ifdef HAVE_POPPLER_0_9
-		case Poppler::Link::JavaScript:
-		{
-			popplerLinkJS = static_cast<const Poppler::LinkJavaScript *>(popplerLink);
-			link = new Okular::ScriptAction( Okular::JavaScript, popplerLinkJS->script() );
-		}
-		break;
+
+#ifdef HAVE_POPPLER_0_20
+        case Poppler::Link::Movie:
+        {
+            deletePopplerLink = false; // we'll delete it inside resolveMovieLinkReferences() after we have resolved all references
+
+            popplerLinkMovie = static_cast<const Poppler::LinkMovie *>( popplerLink );
+
+            Okular::MovieAction::OperationType operation = Okular::MovieAction::Play;
+            switch ( popplerLinkMovie->operation() )
+            {
+                case Poppler::LinkMovie::Play:
+                    operation = Okular::MovieAction::Play;
+                    break;
+                case Poppler::LinkMovie::Stop:
+                    operation = Okular::MovieAction::Stop;
+                    break;
+                case Poppler::LinkMovie::Pause:
+                    operation = Okular::MovieAction::Pause;
+                    break;
+                case Poppler::LinkMovie::Resume:
+                    operation = Okular::MovieAction::Resume;
+                    break;
+            };
+
+            Okular::MovieAction *movieAction = new Okular::MovieAction( operation );
+            movieAction->setNativeId( QVariant::fromValue( popplerLinkMovie ) );
+            link = movieAction;
+        }
+        break;
 #endif
-		
-		case Poppler::Link::Movie:
-			// not implemented
-		break;
-	}
-	
-	return link;
+    }
+
+    if ( deletePopplerLink )
+        delete popplerLink;
+
+    return link;
 }
 
+/**
+ * Note: the function will take ownership of the popplerLink objects.
+ */
 static QLinkedList<Okular::ObjectRect*> generateLinks( const QList<Poppler::Link*> &popplerLinks )
 {
-	QLinkedList<Okular::ObjectRect*> links;
-	foreach(const Poppler::Link *popplerLink, popplerLinks)
-	{
-		QRectF linkArea = popplerLink->linkArea();
-		double nl = linkArea.left(),
-		       nt = linkArea.top(),
-		       nr = linkArea.right(),
-		       nb = linkArea.bottom();
-		// create the rect using normalized coords and attach the Okular::Link to it
-		Okular::ObjectRect * rect = new Okular::ObjectRect( nl, nt, nr, nb, false, Okular::ObjectRect::Action, createLinkFromPopplerLink(popplerLink) );
-		// add the ObjectRect to the container
-		links.push_front( rect );
-	}
-	qDeleteAll(popplerLinks);
-	return links;
+    QLinkedList<Okular::ObjectRect*> links;
+    foreach(const Poppler::Link *popplerLink, popplerLinks)
+    {
+        QRectF linkArea = popplerLink->linkArea();
+        double nl = linkArea.left(),
+               nt = linkArea.top(),
+               nr = linkArea.right(),
+               nb = linkArea.bottom();
+        // create the rect using normalized coords and attach the Okular::Link to it
+        Okular::ObjectRect * rect = new Okular::ObjectRect( nl, nt, nr, nb, false, Okular::ObjectRect::Action, createLinkFromPopplerLink(popplerLink) );
+        // add the ObjectRect to the container
+        links.push_front( rect );
+    }
+    return links;
 }
 
-extern Okular::Annotation* createAnnotationFromPopplerAnnotation( Poppler::Annotation *ann, bool * doDelete );
-
 /** NOTES on threading:
  * internal: thread race prevention is done via the 'docLock' mutex. the
  *           mutex is needed only because we have the asynchronous thread; else
@@ -278,7 +358,7 @@ static KAboutData createAboutData()
          "okular_poppler",
          "okular_poppler",
          ki18n( "PDF Backend" ),
-         "0.5",
+         "0.6",
          ki18n( "A PDF file renderer" ),
          KAboutData::License_GPL,
          ki18n( "© 2005-2008 Albert Astals Cid" )
@@ -302,7 +382,7 @@ PDFGenerator::PDFGenerator( QObject *parent, const QVariantList &args )
     docInfoDirty( true ), docSynopsisDirty( true ),
     docEmbeddedFilesDirty( true ), nextFontPage( 0 ),
     dpiX( 72.0 /*Okular::Utils::dpiX()*/ ), dpiY( 72.0 /*Okular::Utils::dpiY()*/ ),
-    synctex_scanner(0)
+    annotProxy( 0 ), synctex_scanner( 0 )
 {
     setFeature( Threaded );
     setFeature( TextExtraction );
@@ -315,7 +395,7 @@ PDFGenerator::PDFGenerator( QObject *parent, const QVariantList &args )
     if ( Okular::FilePrinter::ps2pdfAvailable() )
         setFeature( PrintToFile );
     setFeature( ReadRawData );
-    
+
 #ifdef HAVE_POPPLER_0_16
     // You only need to do it once not for each of the documents but it is cheap enough
     // so doing it all the time won't hurt either
@@ -444,11 +524,16 @@ bool PDFGenerator::init(QVector<Okular::Page*> & pagesVector, const QString &wal
     pagesVector.resize(pageCount);
     rectsGenerated.fill(false, pageCount);
 
+    annotationsHash.clear();
+
     loadPages(pagesVector, 0, false);
 
     // update the configuration
     reparseConfig();
 
+    // create annotation proxy
+    annotProxy = new PopplerAnnotationProxy( pdfdoc, userMutex() );
+
     // the file has been loaded correctly
     return true;
 }
@@ -457,6 +542,8 @@ bool PDFGenerator::doCloseDocument()
 {
     // remove internal objects
     userMutex()->lock();
+    delete annotProxy;
+    annotProxy = 0;
     delete pdfdoc;
     pdfdoc = 0;
     userMutex()->unlock();
@@ -511,13 +598,11 @@ void PDFGenerator::loadPages(QVector<Okular::Page*> &pagesVector, int rotation,
             if ( tmplink )
             {
                 page->setPageAction( Okular::Page::Opening, createLinkFromPopplerLink( tmplink ) );
-                delete tmplink;
             }
             tmplink = p->action( Poppler::Page::Closing );
             if ( tmplink )
             {
                 page->setPageAction( Okular::Page::Closing, createLinkFromPopplerLink( tmplink ) );
-                delete tmplink;
             }
             page->setDuration( p->duration() );
             page->setLabel( p->label() );
@@ -547,9 +632,9 @@ const Okular::DocumentInfo * PDFGenerator::generateDocumentInfo()
     if ( docInfoDirty )
     {
         userMutex()->lock();
-        
+
         docInfo.set( Okular::DocumentInfo::MimeType, "application/pdf" );
-        
+
         if ( pdfdoc )
         {
             // compile internal structure reading properties from PDFDoc
@@ -700,13 +785,11 @@ Okular::FontInfo::List PDFGenerator::fontsForPage( int page )
         of.setType( convertPopplerFontInfoTypeToOkularFontInfoType( font.type() ) );
         of.setEmbedType( embedTypeForPopplerFontInfo( font) );
         of.setFile( font.file() );
-#ifdef HAVE_POPPLER_0_9
         of.setCanBeExtracted( of.embedType() != Okular::FontInfo::NotEmbedded );
-        
+
         QVariant nativeId;
         nativeId.setValue( font );
         of.setNativeId( nativeId );
-#endif
 
         list.append( of );
     }
@@ -800,21 +883,66 @@ QImage PDFGenerator::image( Okular::PixmapRequest * request )
 
     if ( p && genObjectRects )
     {
-    	// TODO previously we extracted Image type rects too, but that needed porting to poppler
+        // TODO previously we extracted Image type rects too, but that needed porting to poppler
         // and as we are not doing anything with Image type rects i did not port it, have a look at
         // dead gp_outputdev.cpp on image extraction
         page->setObjectRects( generateLinks(p->links()) );
         rectsGenerated[ request->page()->number() ] = true;
+
+        resolveMovieLinkReferences( page );
     }
 
     // 3. UNLOCK [re-enables shared access]
     userMutex()->unlock();
 
     delete p;
-    
+
     return img;
 }
 
+void PDFGenerator::resolveMovieLinkReference( Okular::Action *action, Okular::Page *page )
+{
+#ifdef HAVE_POPPLER_0_20
+    if ( !action )
+        return;
+
+    if ( action->actionType() != Okular::Action::Movie )
+        return;
+
+    Okular::MovieAction *movieAction = static_cast<Okular::MovieAction*>( action );
+
+    const Poppler::LinkMovie *linkMovie = movieAction->nativeId().value<const Poppler::LinkMovie*>();
+
+    QHashIterator<Okular::Annotation*, Poppler::Annotation*> it( annotationsHash );
+    while ( it.hasNext() )
+    {
+        it.next();
+
+        if ( it.key()->subType() == Okular::Annotation::AMovie )
+        {
+            const Poppler::MovieAnnotation *movieAnnotation = static_cast<const Poppler::MovieAnnotation*>( it.value() );
+
+            if ( linkMovie->isReferencedAnnotation( movieAnnotation ) )
+            {
+                movieAction->setAnnotation( static_cast<Okular::MovieAnnotation*>( it.key() ) );
+                movieAction->setNativeId( QVariant() );
+                delete linkMovie; // delete the associated Poppler::LinkMovie object, it's not needed anymore
+                break;
+            }
+        }
+    }
+#endif
+}
+
+void PDFGenerator::resolveMovieLinkReferences( Okular::Page *page )
+{
+    resolveMovieLinkReference( const_cast<Okular::Action*>( page->pageAction( Okular::Page::Opening ) ), page );
+    resolveMovieLinkReference( const_cast<Okular::Action*>( page->pageAction( Okular::Page::Closing ) ), page );
+
+    foreach ( Okular::FormField *field, page->formFields() )
+        resolveMovieLinkReference( field->activationAction(), page );
+}
+
 Okular::TextPage* PDFGenerator::textPage( Okular::Page *page )
 {
 #ifdef PDFGENERATOR_DEBUG
@@ -836,7 +964,7 @@ Okular::TextPage* PDFGenerator::textPage( Okular::Page *page )
 
         delete pp;
     }
-    else   
+    else
     {
         pageWidth = defaultPageWidth;
         pageHeight = defaultPageHeight;
@@ -849,13 +977,8 @@ Okular::TextPage* PDFGenerator::textPage( Okular::Page *page )
 
 void PDFGenerator::requestFontData(const Okular::FontInfo &font, QByteArray *data)
 {
-#ifdef HAVE_POPPLER_0_9
     Poppler::FontInfo fi = font.nativeId().value<Poppler::FontInfo>();
     *data = pdfdoc->fontData(fi);
-#else
-    Q_UNUSED( font )
-    Q_UNUSED( data )
-#endif
 }
 
 #define DUMMY_QPRINTER_COPY
@@ -926,9 +1049,11 @@ bool PDFGenerator::print( QPrinter& printer )
         pstitle = document()->currentDocument().fileName();
     }
 
+    bool printAnnots = true;
     bool forceRasterize = false;
     if ( pdfOptionsPage )
     {
+        printAnnots = pdfOptionsPage->printAnnots();
         forceRasterize = pdfOptionsPage->printForceRaster();
     }
 
@@ -947,6 +1072,11 @@ bool PDFGenerator::print( QPrinter& printer )
     psConverter->setForceRasterize(forceRasterize);
     psConverter->setTitle(pstitle);
 
+#ifdef HAVE_POPPLER_0_20
+    if (!printAnnots)
+        psConverter->setPSOptions(psConverter->psOptions() | Poppler::PSConverter::HideAnnotations );
+#endif
+
     userMutex()->lock();
     if (psConverter->convert())
     {
@@ -990,7 +1120,7 @@ QVariant PDFGenerator::metaData( const QString & key, const QVariant & option )
         Okular::DocumentViewport viewport;
         QString optionString = option.toString();
 
-        // if option starts with "src:" assume that we are handling a 
+        // if option starts with "src:" assume that we are handling a
         // source reference
         if ( optionString.startsWith( "src:", Qt::CaseInsensitive ) )
         {
@@ -1025,13 +1155,11 @@ QVariant PDFGenerator::metaData( const QString & key, const QVariant & option )
         if ( pdfdoc->pageMode() == Poppler::Document::UseOutlines )
             return true;
     }
-#ifdef HAVE_POPPLER_0_9
     else if ( key == "DocumentScripts" && option.toString() == "JavaScript" )
     {
         QMutexLocker ml(userMutex());
         return pdfdoc->scripts();
     }
-#endif
     return QVariant();
 }
 
@@ -1130,20 +1258,14 @@ inline void append (Okular::TextPage* ktp,
     const QString &s, double l, double b, double r, double t)
 {
 //    kWarning(PDFDebug).nospace() << "text: " << s << " at (" << l << "," << t << ")x(" << r <<","<<b<<")";
-                ktp->append( s ,
-                    new Okular::NormalizedRect(
-                    l,
-                    t,
-                    r,
-                    b
-                    ));
+    ktp->append(s, new Okular::NormalizedRect(l, t, r, b));
 }
 
 Okular::TextPage * PDFGenerator::abstractTextPage(const QList<Poppler::TextBox*> &text, double height, double width,int rot)
-{    
+{
     Q_UNUSED(rot);
     Okular::TextPage* ktp=new Okular::TextPage;
-    Poppler::TextBox *next; 
+    Poppler::TextBox *next;
 #ifdef PDFGENERATOR_DEBUG
     kDebug(PDFDebug) << "getting text page in generator pdf - rotation:" << rot;
 #endif
@@ -1172,7 +1294,7 @@ Okular::TextPage * PDFGenerator::abstractTextPage(const QList<Poppler::TextBox*>
                 s = c;
                 addChar = true;
             }
-            
+
             if (addChar)
             {
                 QRectF charBBox = word->charBoundingBox(textBoxChar);
@@ -1193,7 +1315,7 @@ Okular::TextPage * PDFGenerator::abstractTextPage(const QList<Poppler::TextBox*>
             // vertically or horizontally aligned
             QRectF wordBBox = word->boundingBox();
             QRectF nextWordBBox = next->boundingBox();
-            append(ktp, " ", 
+            append(ktp, " ",
                      wordBBox.right()/width,
                      wordBBox.bottom()/height,
                      nextWordBBox.left()/width,
@@ -1228,7 +1350,7 @@ void PDFGenerator::addSynopsisChildren( QDomNode * parent, QDomNode * parentDest
         if (!e.attribute("DestinationURI").isNull()) item.setAttribute("URL", e.attribute("DestinationURI"));
 
         // descend recursively and advance to the next node
-        if ( e.hasChildNodes() ) addSynopsisChildren( &n, &	item );
+        if ( e.hasChildNodes() ) addSynopsisChildren( &n, &    item );
         n = n.nextSibling();
     }
 }
@@ -1236,6 +1358,10 @@ void PDFGenerator::addSynopsisChildren( QDomNode * parent, QDomNode * parentDest
 void PDFGenerator::addAnnotations( Poppler::Page * popplerPage, Okular::Page * page )
 {
     QList<Poppler::Annotation*> popplerAnnotations = popplerPage->annotations();
+
+    // Reverse the list so that the z-order of Poppler/PDF matches the z-order used by Okular
+    std::reverse(popplerAnnotations.begin(), popplerAnnotations.end());
+
     foreach(Poppler::Annotation *a, popplerAnnotations)
     {
         //a->window.width = (int)(page->width() * a->window.width);
@@ -1283,6 +1409,9 @@ void PDFGenerator::addAnnotations( Poppler::Page * popplerPage, Okular::Page * p
             // explicitly mark as external
             newann->setFlags( newann->flags() | Okular::Annotation::External );
             page->addAnnotation(newann);
+
+            if ( !doDelete )
+                annotationsHash.insert( newann, a );
         }
         if ( doDelete )
             delete a;
@@ -1565,7 +1694,7 @@ void PDFGenerator::fillViewportFromSourceReference( Okular::DocumentViewport & v
     // Extract the file name and the numeral part from the reference string.
     // This will fail if Filename starts with a digit.
     QString name, lineString;
-    // Remove "src:". Presence of substring has been checked before this 
+    // Remove "src:". Presence of substring has been checked before this
     // function is called.
     name = reference.mid( 4 );
     // split
@@ -1580,18 +1709,18 @@ void PDFGenerator::fillViewportFromSourceReference( Okular::DocumentViewport & v
     // Remove spaces.
     name = name.trimmed();
     lineString = lineString.trimmed();
-    // Convert line to integer. 
+    // Convert line to integer.
     bool ok;
     int line = lineString.toInt( &ok );
     if (!ok) line = -1;
-    
+
     // Use column == -1 for now.
     if( synctex_display_query( synctex_scanner, name.toLatin1(), line, -1 ) > 0 )
     {
         synctex_node_t node;
         // For now use the first hit. Could possibly be made smarter
         // in case there are multiple hits.
-        while( ( node = synctex_next_result( synctex_scanner ) ) ) 
+        while( ( node = synctex_next_result( synctex_scanner ) ) )
         {
             // TeX pages start at 1.
             viewport.pageNumber = synctex_node_page( node ) - 1;
@@ -1605,7 +1734,7 @@ void PDFGenerator::fillViewportFromSourceReference( Okular::DocumentViewport & v
             viewport.rePos.normalizedY = ( py + 0.5 ) / document()->page(viewport.pageNumber)->height();
             viewport.rePos.enabled = true;
             viewport.rePos.pos = Okular::DocumentViewport::Center;
-           
+
             return;
         }
     }
@@ -1625,7 +1754,11 @@ bool PDFGenerator::supportsOption( SaveOption option ) const
     switch ( option )
     {
         case SaveChanges:
-            return true;
+        {
+            QMutexLocker locker( userMutex() );
+            // Saving files with /Encrypt is not supported
+            return pdfdoc->isEncrypted() ? false : true;
+        }
         default: ;
     }
     return false;
@@ -1665,6 +1798,11 @@ bool PDFGenerator::save( const QString &fileName, SaveOptions options, QString *
     return success;
 }
 
+Okular::AnnotationProxy* PDFGenerator::annotationProxy() const
+{
+    return annotProxy;
+}
+
 #include "generator_pdf.moc"
 
 /* kate: replace-tabs on; indent-width 4; */
