@@ -151,8 +151,25 @@ class FileKeeper
         std::FILE * m_handle;
 };
 
-K_PLUGIN_FACTORY( okularPartFactory, registerPlugin< Okular::Part >(); )
-K_EXPORT_PLUGIN( okularPartFactory( okularAboutData( "okular", I18N_NOOP( "Okular" ) ) ) )
+Okular::PartFactory::PartFactory()
+: KPluginFactory(okularAboutData( "okular", I18N_NOOP( "Okular" ) ))
+{
+}
+
+Okular::PartFactory::~PartFactory()
+{
+}
+
+QObject *Okular::PartFactory::create(const char *iface, QWidget *parentWidget, QObject *parent, const QVariantList &args, const QString &keyword)
+{
+    Q_UNUSED ( keyword );
+
+    Okular::Part *object = new Okular::Part( parentWidget, parent, args, componentData() );
+    object->setReadWrite( QLatin1String(iface) == QLatin1String("KParts::ReadWritePart") );
+    return object;
+}
+
+K_EXPORT_PLUGIN( Okular::PartFactory() )
 
 static QAction* actionForExportFormat( const Okular::ExportFormat& format, QObject *parent = 0 )
 {
@@ -264,8 +281,9 @@ namespace Okular
 
 Part::Part(QWidget *parentWidget,
 QObject *parent,
-const QVariantList &args )
-: KParts::ReadOnlyPart(parent),
+const QVariantList &args,
+KComponentData componentData )
+: KParts::ReadWritePart(parent),
 m_tempfile( 0 ), m_fileWasRemoved( false ), m_showMenuBarAction( 0 ), m_showFullScreenAction( 0 ), m_actionsSearched( false ),
 m_cliPresentation(false), m_embedMode(detectEmbedMode(parentWidget, parent, args)), m_generatorGuiClient(0), m_keeper( 0 )
 {
@@ -300,7 +318,7 @@ m_cliPresentation(false), m_embedMode(detectEmbedMode(parentWidget, parent, args
     new OkularLiveConnectExtension( this );
 
     // we need an instance
-    setComponentData(okularPartFactory::componentData());
+    setComponentData( componentData );
 
     GuiUtils::addIconLoader( iconLoader() );
 
@@ -410,19 +428,21 @@ m_cliPresentation(false), m_embedMode(detectEmbedMode(parentWidget, parent, args
     bottomBarLayout->setSpacing( 0 );
     bottomBarLayout->addWidget( m_pageSizeLabel->antiWidget() );
     bottomBarLayout->addItem( new QSpacerItem( 5, 5, QSizePolicy::Expanding, QSizePolicy::Minimum ) );
-    m_miniBar = new MiniBar( m_bottomBar, m_document );
+    m_miniBarLogic = new MiniBarLogic( this, m_document );
+    m_miniBar = new MiniBar( m_bottomBar, m_miniBarLogic );
     bottomBarLayout->addWidget( m_miniBar );
     bottomBarLayout->addItem( new QSpacerItem( 5, 5, QSizePolicy::Expanding, QSizePolicy::Minimum ) );
     bottomBarLayout->addWidget( m_pageSizeLabel );
     rightLayout->addWidget( m_bottomBar );
 
+    m_pageNumberTool = new MiniBar( 0, m_miniBarLogic );
+
     connect( m_findBar, SIGNAL(forwardKeyPressEvent(QKeyEvent*)), m_pageView, SLOT(externalKeyPressEvent(QKeyEvent*)));
     connect( m_miniBar, SIGNAL(forwardKeyPressEvent(QKeyEvent*)), m_pageView, SLOT(externalKeyPressEvent(QKeyEvent*)));
+    connect( m_pageNumberTool, SIGNAL(forwardKeyPressEvent(QKeyEvent*)), m_pageView, SLOT(externalKeyPressEvent(QKeyEvent*)));
 
-    connect( m_reviewsWidget, SIGNAL(setAnnotationWindow(Okular::Annotation*)),
-        m_pageView, SLOT(setAnnotationWindow(Okular::Annotation*)) );
-    connect( m_reviewsWidget, SIGNAL(removeAnnotationWindow(Okular::Annotation*)),
-        m_pageView, SLOT(removeAnnotationWindow(Okular::Annotation*)) );
+    connect( m_reviewsWidget, SIGNAL(openAnnotationWindow(Okular::Annotation*,int)),
+        m_pageView, SLOT(openAnnotationWindow(Okular::Annotation*,int)) );
 
     // add document observers
     m_document->addObserver( this );
@@ -430,7 +450,7 @@ m_cliPresentation(false), m_embedMode(detectEmbedMode(parentWidget, parent, args
     m_document->addObserver( m_pageView );
     m_document->registerView( m_pageView );
     m_document->addObserver( m_toc );
-    m_document->addObserver( m_miniBar );
+    m_document->addObserver( m_miniBarLogic );
 #ifdef OKULAR_ENABLE_MINIBAR
     m_document->addObserver( m_progressWidget );
 #endif
@@ -516,6 +536,7 @@ void Part::setupViewerActions()
     m_gotoPage->setShortcut( QKeySequence(Qt::CTRL + Qt::Key_G) );
     // dirty way to activate gotopage when pressing miniBar's button
     connect( m_miniBar, SIGNAL(gotoPage()), m_gotoPage, SLOT(trigger()) );
+    connect( m_pageNumberTool, SIGNAL(gotoPage()), m_gotoPage, SLOT(trigger()) );
 
     m_prevPage = KStandardAction::prior(this, SLOT(slotPreviousPage()), ac);
     m_prevPage->setIconText( i18nc( "Previous page", "Previous" ) );
@@ -524,6 +545,7 @@ void Part::setupViewerActions()
     m_prevPage->setShortcut( 0 );
     // dirty way to activate prev page when pressing miniBar's button
     connect( m_miniBar, SIGNAL(prevPage()), m_prevPage, SLOT(trigger()) );
+    connect( m_pageNumberTool, SIGNAL(prevPage()), m_prevPage, SLOT(trigger()) );
 #ifdef OKULAR_ENABLE_MINIBAR
     connect( m_progressWidget, SIGNAL(prevPage()), m_prevPage, SLOT(trigger()) );
 #endif
@@ -535,6 +557,7 @@ void Part::setupViewerActions()
     m_nextPage->setShortcut( 0 );
     // dirty way to activate next page when pressing miniBar's button
     connect( m_miniBar, SIGNAL(nextPage()), m_nextPage, SLOT(trigger()) );
+    connect( m_pageNumberTool, SIGNAL(nextPage()), m_nextPage, SLOT(trigger()) );
 #ifdef OKULAR_ENABLE_MINIBAR
     connect( m_progressWidget, SIGNAL(nextPage()), m_nextPage, SLOT(trigger()) );
 #endif
@@ -560,19 +583,19 @@ void Part::setupViewerActions()
     m_renameBookmark = ac->addAction("rename_bookmark");
     m_renameBookmark->setText(i18n( "Rename Bookmark" ));
     m_renameBookmark->setIcon(KIcon( "edit-rename" ));
-    m_renameBookmark->setWhatsThis( i18n( "Rename the current page bookmark" ) );
-    connect( m_renameBookmark, SIGNAL(triggered()), this, SLOT(slotRenameCurrentPageBookmark()) );
+    m_renameBookmark->setWhatsThis( i18n( "Rename the current bookmark" ) );
+    connect( m_renameBookmark, SIGNAL(triggered()), this, SLOT(slotRenameCurrentViewportBookmark()) );
 
     m_prevBookmark = ac->addAction("previous_bookmark");
     m_prevBookmark->setText(i18n( "Previous Bookmark" ));
     m_prevBookmark->setIcon(KIcon( "go-up-search" ));
-    m_prevBookmark->setWhatsThis( i18n( "Go to the previous bookmarked page" ) );
+    m_prevBookmark->setWhatsThis( i18n( "Go to the previous bookmark" ) );
     connect( m_prevBookmark, SIGNAL(triggered()), this, SLOT(slotPreviousBookmark()) );
 
     m_nextBookmark = ac->addAction("next_bookmark");
     m_nextBookmark->setText(i18n( "Next Bookmark" ));
     m_nextBookmark->setIcon(KIcon( "go-down-search" ));
-    m_nextBookmark->setWhatsThis( i18n( "Go to the next bookmarked page" ) );
+    m_nextBookmark->setWhatsThis( i18n( "Go to the next bookmark" ) );
     connect( m_nextBookmark, SIGNAL(triggered()), this, SLOT(slotNextBookmark()) );
 
     m_copy = 0;
@@ -657,6 +680,10 @@ void Part::setupViewerActions()
     ac->addAction("close_find_bar", m_closeFindBar);
     connect(m_closeFindBar, SIGNAL(triggered()), this, SLOT(slotHideFindBar()));
     widget()->addAction(m_closeFindBar);
+
+    KAction *pageno = new KAction( i18n( "Page Number" ), ac );
+    pageno->setDefaultWidget( m_pageNumberTool );
+    ac->addAction( "page_number", pageno );
 }
 
 void Part::setViewerShortcuts()
@@ -777,6 +804,8 @@ Part::~Part()
     delete m_pageView;
     delete m_thumbnailList;
     delete m_miniBar;
+    delete m_pageNumberTool;
+    delete m_miniBarLogic;
     delete m_bottomBar;
 #ifdef OKULAR_ENABLE_MINIBAR
     delete m_progressWidget;
@@ -1014,6 +1043,9 @@ void Part::notifyViewportChanged( bool /*smoothMove*/ )
 
 void Part::notifyPageChanged( int page, int flags )
 {
+    if ( flags & Okular::DocumentObserver::NeedSaveAs )
+        setModified();
+
     if ( !(flags & Okular::DocumentObserver::Bookmark ) )
         return;
 
@@ -1247,6 +1279,10 @@ bool Part::openFile()
 
 bool Part::openUrl(const KUrl &_url)
 {
+    // Close current document if any
+    if ( !closeUrl() )
+        return false;
+
     KUrl url( _url );
     if ( url.hasHTMLRef() )
     {
@@ -1270,7 +1306,7 @@ bool Part::openUrl(const KUrl &_url)
     }
 
     // this calls in sequence the 'closeUrl' and 'openFile' methods
-    bool openOk = KParts::ReadOnlyPart::openUrl( url );
+    bool openOk = KParts::ReadWritePart::openUrl( url );
 
     if ( openOk )
     {
@@ -1286,9 +1322,36 @@ bool Part::openUrl(const KUrl &_url)
     return openOk;
 }
 
-
-bool Part::closeUrl()
+bool Part::queryClose()
 {
+    if ( !isReadWrite() || !isModified() )
+        return true;
+
+    const int res = KMessageBox::warningYesNoCancel( widget(),
+                        i18n( "Do you want to save your annotation changes or discard them?" ),
+                        i18n( "Close Document" ),
+                        KStandardGuiItem::saveAs(),
+                        KStandardGuiItem::discard() );
+
+    switch ( res )
+    {
+        case KMessageBox::Yes: // Save as
+            slotSaveFileAs();
+            return !isModified(); // Only allow closing if file was really saved
+        case KMessageBox::No: // Discard
+            return true;
+        default: // Cancel
+            return false;
+    }
+}
+
+bool Part::closeUrl(bool promptToSave)
+{
+    if ( promptToSave && !queryClose() )
+        return false;
+
+    setModified( false );
+
     if (!m_temporaryLocalFile.isNull() && m_temporaryLocalFile != localFilePath())
     {
         QFile::remove( m_temporaryLocalFile );
@@ -1347,17 +1410,22 @@ bool Part::closeUrl()
 #ifdef OKULAR_KEEP_FILE_OPEN
     m_keeper->close();
 #endif
-    bool r = KParts::ReadOnlyPart::closeUrl();
+    bool r = KParts::ReadWritePart::closeUrl();
     setUrl(KUrl());
 
     return r;
 }
 
+bool Part::closeUrl()
+{
+    return closeUrl( true );
+}
+
 void Part::guiActivateEvent(KParts::GUIActivateEvent *event)
 {
     updateViewActions();
 
-    KParts::ReadOnlyPart::guiActivateEvent(event);
+    KParts::ReadWritePart::guiActivateEvent(event);
 }
 
 void Part::close()
@@ -1462,7 +1530,12 @@ void Part::slotDoFileDirty()
     }
 
     // close and (try to) reopen the document
-    if ( KParts::ReadOnlyPart::openUrl( url() ) )
+    KUrl oldUrl = url();
+
+    if ( !closeUrl() )
+        return;
+
+    if ( KParts::ReadWritePart::openUrl( oldUrl ) )
     {
         // on successful opening, restore the previous viewport
         if ( m_viewportDirty.pageNumber >= (int) m_document->pages() )
@@ -1586,7 +1659,7 @@ void Part::updateBookmarksActions()
     if ( opened )
     {
         m_addBookmark->setEnabled( true );
-        if ( m_document->bookmarkManager()->isBookmarked( m_document->currentPage() ) )
+        if ( m_document->bookmarkManager()->isBookmarked( m_document->viewport() ) )
         {
             m_addBookmark->setText( i18n( "Remove Bookmark" ) );
             m_addBookmark->setIcon( KIcon( "edit-delete-bookmark" ) );
@@ -1736,23 +1809,23 @@ void Part::slotHistoryNext()
 
 void Part::slotAddBookmark()
 {
-    uint current = m_document->currentPage();
-    if ( m_document->bookmarkManager()->isBookmarked( current ) )
+    DocumentViewport vp = m_document->viewport();
+    if ( m_document->bookmarkManager()->isBookmarked( vp ) )
     {
-        m_document->bookmarkManager()->removeBookmark( current );
+        m_document->bookmarkManager()->removeBookmark( vp );
     }
     else
     {
-        m_document->bookmarkManager()->addBookmark( current );
+        m_document->bookmarkManager()->addBookmark( vp );
     }
 }
 
-void Part::slotRenameBookmark( int page )
+void Part::slotRenameBookmark( const DocumentViewport &viewport )
 {
-    Q_ASSERT(m_document->bookmarkManager()->isBookmarked( page ));
-    if ( m_document->bookmarkManager()->isBookmarked( page ) )
+    Q_ASSERT(m_document->bookmarkManager()->isBookmarked( viewport ));
+    if ( m_document->bookmarkManager()->isBookmarked( viewport ) )
     {
-        KBookmark bookmark = m_document->bookmarkManager()->bookmark( page );
+        KBookmark bookmark = m_document->bookmarkManager()->bookmark( viewport );
         const QString newName = KInputDialog::getText( i18n( "Rename Bookmark" ), i18n( "Enter the new name of the bookmark:" ), bookmark.fullText(), 0, widget());
         if (!newName.isEmpty())
         {
@@ -1767,13 +1840,14 @@ void Part::slotRenameBookmarkFromMenu()
     Q_ASSERT( action );
     if ( action )
     {
-        slotRenameBookmark( action->data().toInt() );
+        DocumentViewport vp( action->data().toString() );
+        slotRenameBookmark( vp );
     }
 }
 
-void Part::slotRenameCurrentPageBookmark()
+void Part::slotRenameCurrentViewportBookmark()
 {
-    slotRenameBookmark( m_document->currentPage() );
+    slotRenameBookmark( m_document->viewport() );
 }
 
 void Part::slotAboutToShowContextMenu(KMenu * /*menu*/, QAction *action, QMenu *contextMenu)
@@ -1791,44 +1865,31 @@ void Part::slotAboutToShowContextMenu(KMenu * /*menu*/, QAction *action, QMenu *
         QAction *separatorAction = contextMenu->addSeparator();
         separatorAction->setObjectName("OkularPrivateRenameBookmarkActions");
         QAction *renameAction = contextMenu->addAction( KIcon( "edit-rename" ), i18n( "Rename this Bookmark" ), this, SLOT(slotRenameBookmarkFromMenu()) );
-        renameAction->setData(ba->property("pageNumber").toInt() - 1); // These page numbers are indexed starting on 1
+        renameAction->setData(ba->property("htmlRef").toString());
         renameAction->setObjectName("OkularPrivateRenameBookmarkActions");
     }
 }
 
 void Part::slotPreviousBookmark()
 {
-    uint current = m_document->currentPage();
-    // we are at the first page
-    if ( current == 0 )
-        return;
+    const KBookmark bookmark = m_document->bookmarkManager()->previousBookmark( m_document->viewport() );
 
-    for ( int i = current - 1; i >= 0; --i )
+    if ( !bookmark.isNull() )
     {
-        if ( m_document->bookmarkManager()->isBookmarked( i ) )
-        {
-            m_document->setViewportPage( i );
-            break;
-        }
+        DocumentViewport vp( bookmark.url().htmlRef() );
+        m_document->setViewport( vp );
     }
 }
 
 
 void Part::slotNextBookmark()
 {
-    uint current = m_document->currentPage();
-    uint pages = m_document->pages();
-    // we are at the last page
-    if ( current == pages )
-        return;
+    const KBookmark bookmark = m_document->bookmarkManager()->nextBookmark( m_document->viewport() );
 
-    for ( uint i = current + 1; i < pages; ++i )
+    if ( !bookmark.isNull() )
     {
-        if ( m_document->bookmarkManager()->isBookmarked( i ) )
-        {
-            m_document->setViewportPage( i );
-            break;
-        }
+        DocumentViewport vp( bookmark.url().htmlRef() );
+        m_document->setViewport( vp );
     }
 }
 
@@ -1865,6 +1926,11 @@ void Part::slotFindPrev()
         m_findBar->findPrev();
 }
 
+bool Part::saveFile()
+{
+    kDebug() << "Okular part doesn't support saving the file in the location from which it was opened";
+    return false;
+}
 
 void Part::slotSaveFileAs()
 {
@@ -1877,12 +1943,17 @@ void Part::slotSaveFileAs()
     if ( !saveUrl.isValid() || saveUrl.isEmpty() )
         return;
 
+    saveAs( saveUrl );
+}
+
+bool Part::saveAs( const KUrl & saveUrl )
+{
     KTemporaryFile tf;
     QString fileName;
     if ( !tf.open() )
     {
         KMessageBox::information( widget(), i18n("Could not open the temporary file for saving." ) );
-            return;
+            return false;
     }
     fileName = tf.fileName();
     tf.close();
@@ -1898,12 +1969,18 @@ void Part::slotSaveFileAs()
         {
             KMessageBox::information( widget(), i18n("File could not be saved in '%1'. %2", fileName, errorText ) );
         }
-        return;
+        return false;
     }
 
     KIO::Job *copyJob = KIO::file_copy( fileName, saveUrl, -1, KIO::Overwrite );
     if ( !KIO::NetAccess::synchronousRun( copyJob, widget() ) )
+    {
         KMessageBox::information( widget(), i18n("File could not be saved in '%1'. Try to save it to another location.", saveUrl.prettyUrl() ) );
+        return false;
+    }
+
+    setModified( false );
+    return true;
 }
 
 
@@ -2108,7 +2185,7 @@ void Part::slotShowMenu(const Okular::Page *page, const QPoint &point)
     if (page)
     {
         popup->addTitle( i18n( "Page %1", page->number() + 1 ) );
-        if ( m_document->bookmarkManager()->isBookmarked( page->number() ) )
+        if ( m_document->bookmarkManager()->isBookmarked( m_document->viewport() ) )
             removeBookmark = popup->addAction( KIcon("edit-delete-bookmark"), i18n("Remove Bookmark") );
         else
             addBookmark = popup->addAction( KIcon("bookmark-new"), i18n("Add Bookmark") );
@@ -2140,8 +2217,8 @@ void Part::slotShowMenu(const Okular::Page *page, const QPoint &point)
         QAction *res = popup->exec(point);
         if (res)
         {
-            if (res == addBookmark) m_document->bookmarkManager()->addBookmark( page->number() );
-            else if (res == removeBookmark) m_document->bookmarkManager()->removeBookmark( page->number() );
+            if (res == addBookmark) m_document->bookmarkManager()->addBookmark( m_document->viewport() );
+            else if (res == removeBookmark) m_document->bookmarkManager()->removeBookmark( m_document->viewport() );
             else if (res == fitPageWidth) m_pageView->fitPageWidth( page->number() );
         }
     }
@@ -2578,6 +2655,12 @@ void Part::updateAboutBackendAction()
     }
 }
 
+void Part::setReadWrite(bool readwrite)
+{
+    m_document->setAnnotationEditingEnabled( readwrite );
+    ReadWritePart::setReadWrite( readwrite );
+}
+
 } // namespace Okular
 
 #include "part.moc"
