@@ -17,6 +17,7 @@
 #define _PART_H_
 
 #include <kparts/part.h>
+#include <kpluginfactory.h>
 #include <qicon.h>
 #include <qlist.h>
 #include <qpointer.h>
@@ -56,6 +57,7 @@ class SearchWidget;
 class Sidebar;
 class TOC;
 class MiniBar;
+class MiniBarLogic;
 class FileKeeper;
 class Reviews;
 class BookmarkList;
@@ -88,7 +90,7 @@ enum EmbedMode
  * @author Wilco Greven <greven@kde.org>
  * @version 0.2
  */
-class Part : public KParts::ReadOnlyPart, public Okular::DocumentObserver, public KDocumentViewer, public Okular::ViewerInterface
+class Part : public KParts::ReadWritePart, public Okular::DocumentObserver, public KDocumentViewer, public Okular::ViewerInterface
 {
     Q_OBJECT
     Q_CLASSINFO("D-Bus Interface", "org.kde.okular")
@@ -103,7 +105,7 @@ class Part : public KParts::ReadOnlyPart, public Okular::DocumentObserver, publi
          * which config file should be used by adding a string containing "ConfigFileName=<file name>"
          * to 'args'.
          **/
-        Part(QWidget* parentWidget, QObject* parent, const QVariantList& args);
+        Part(QWidget* parentWidget, QObject* parent, const QVariantList& args, KComponentData componentData);
 
         // Destructor
         ~Part();
@@ -150,11 +152,17 @@ class Part : public KParts::ReadOnlyPart, public Okular::DocumentObserver, publi
         void viewerMenuStateChange(bool enabled);
 
     protected:
-        // reimplemented from KParts::ReadOnlyPart
+        // reimplemented from KParts::ReadWritePart
         bool openFile();
         bool openUrl(const KUrl &url);
-        bool closeUrl();
         void guiActivateEvent(KParts::GUIActivateEvent *event);
+    public:
+        bool saveFile();
+        bool queryClose();
+        bool closeUrl();
+        bool closeUrl(bool promptToSave);
+        void setReadWrite(bool readwrite);
+        bool saveAs(const KUrl & saveUrl);
 
     protected slots:
         // connected to actions
@@ -165,7 +173,7 @@ class Part : public KParts::ReadOnlyPart, public Okular::DocumentObserver, publi
         void slotHistoryNext();
         void slotAddBookmark();
         void slotRenameBookmarkFromMenu();
-        void slotRenameCurrentPageBookmark();
+        void slotRenameCurrentViewportBookmark();
         void slotAboutToShowContextMenu(KMenu *menu, QAction *action, QMenu *contextMenu);
         void slotPreviousBookmark();
         void slotNextBookmark();
@@ -221,7 +229,7 @@ class Part : public KParts::ReadOnlyPart, public Okular::DocumentObserver, publi
         void rebuildBookmarkMenu( bool unplugActions = true );
         void updateAboutBackendAction();
         void unsetDummyMode();
-        void slotRenameBookmark( int page );
+        void slotRenameBookmark( const DocumentViewport &viewport );
 
         KTemporaryFile *m_tempfile;
 
@@ -238,7 +246,9 @@ class Part : public KParts::ReadOnlyPart, public Okular::DocumentObserver, publi
         QPointer<ThumbnailList> m_thumbnailList;
         QPointer<PageView> m_pageView;
         QPointer<TOC> m_toc;
+        QPointer<MiniBarLogic> m_miniBarLogic;
         QPointer<MiniBar> m_miniBar;
+        QPointer<MiniBar> m_pageNumberTool;
         QPointer<QWidget> m_bottomBar;
         QPointer<PresentationWidget> m_presentationWidget;
         QPointer<ProgressWidget> m_progressWidget;
@@ -316,6 +326,18 @@ class Part : public KParts::ReadOnlyPart, public Okular::DocumentObserver, publi
         void slotHandleActivatedSourceReference(const QString& absFileName, int line, int col, bool *handled);
 };
 
+class PartFactory : public KPluginFactory
+{
+    Q_OBJECT
+
+    public:
+        PartFactory();
+        virtual ~PartFactory();
+
+    protected:
+        virtual QObject *create(const char *iface, QWidget *parentWidget, QObject *parent, const QVariantList &args, const QString &keyword);
+};
+
 }
 
 #endif
