@@ -31,6 +31,7 @@
 
 // local includes
 #include "core/annotations.h"
+#include "core/document.h"
 #include "guiutils.h"
 #include "latexrenderer.h"
 #include <core/utils.h>
@@ -80,7 +81,7 @@ public:
         dateLabel->setCursor( Qt::SizeAllCursor );
         buttonlay->addWidget( dateLabel );
         CloseButton * close = new CloseButton( this );
-        connect( close, SIGNAL(clicked()), parent, SLOT(hide()) );
+        connect( close, SIGNAL(clicked()), parent, SLOT(close()) );
         buttonlay->addWidget( close );
         // option button row
         QHBoxLayout * optionlay = new QHBoxLayout();
@@ -182,11 +183,14 @@ private:
 
 
 // Qt::SubWindow is needed to make QSizeGrip work
-AnnotWindow::AnnotWindow( QWidget * parent, Okular::Annotation * annot)
-    : QFrame( parent, Qt::SubWindow ), m_annot( annot )
+AnnotWindow::AnnotWindow( QWidget * parent, Okular::Annotation * annot, Okular::Document *document, int page )
+    : QFrame( parent, Qt::SubWindow ), m_annot( annot ), m_document( document ), m_page( page )
 {
     setAutoFillBackground( true );
     setFrameStyle( Panel | Raised );
+    setAttribute( Qt::WA_DeleteOnClose );
+
+    const bool canEditAnnotation = m_document->canModifyPageAnnotation( annot );
 
     textEdit = new KTextEdit( this );
     textEdit->setAcceptRichText( false );
@@ -195,6 +199,9 @@ AnnotWindow::AnnotWindow( QWidget * parent, Okular::Annotation * annot)
     connect(textEdit,SIGNAL(textChanged()),
             this,SLOT(slotsaveWindowText()));
 
+    if (!canEditAnnotation)
+        textEdit->setReadOnly(true);
+
     m_latexRenderer = new GuiUtils::LatexRenderer();
     emit containsLatex( GuiUtils::LatexRenderer::mightContainLatex( GuiUtils::contents( m_annot ) ) );
 
@@ -238,6 +245,11 @@ void AnnotWindow::reloadInfo()
     m_title->setDate( m_annot->modificationDate() );
 }
 
+Okular::Annotation* AnnotWindow::annotation() const
+{
+    return m_annot;
+}
+
 void AnnotWindow::showEvent( QShowEvent * event )
 {
     QFrame::showEvent( event );
@@ -269,27 +281,47 @@ void AnnotWindow::slotOptionBtn()
 void AnnotWindow::slotsaveWindowText()
 {
     const QString newText = textEdit->toPlainText();
+    bool appearanceChanged = false;
 
-    // 1. window text
+    // Set window text
     if ( !m_annot->window().text().isEmpty() )
     {
         m_annot->window().setText( newText );
         return;
     }
-    // 2. if Text and InPlace, the inplace text
-    if ( m_annot->subType() == Okular::Annotation::AText )
+
+    // Handle special cases
+    switch ( m_annot->subType() )
     {
-        Okular::TextAnnotation * txtann = static_cast< Okular::TextAnnotation * >( m_annot );
-        if ( txtann->textType() == Okular::TextAnnotation::InPlace )
+        // If it's an in-place TextAnnotation, set the inplace text
+        case Okular::Annotation::AText:
         {
-            txtann->setInplaceText( newText );
-            return;
+            Okular::TextAnnotation * txtann = static_cast< Okular::TextAnnotation * >( m_annot );
+            if ( txtann->textType() == Okular::TextAnnotation::InPlace )
+            {
+                txtann->setInplaceText( newText );
+                appearanceChanged = true;
+            }
+            break;
         }
+        // If it's a LineAnnotation, check if caption text is visible
+        case Okular::Annotation::ALine:
+        {
+            Okular::LineAnnotation * lineann = static_cast< Okular::LineAnnotation * >( m_annot );
+            if ( lineann->showCaption() )
+                appearanceChanged = true;
+            break;
+        }
+        default:
+            break;
     }
 
-    // 3. contents
+    // Set contents
     m_annot->setContents( newText );
 
+    // Tell the document
+    m_document->modifyPageAnnotation( m_page, m_annot, appearanceChanged );
+
     emit containsLatex( GuiUtils::LatexRenderer::mightContainLatex( newText ) );
 }
 
