@@ -37,6 +37,7 @@ class Action;
 class Document;
 class DocumentViewport;
 class Annotation;
+class MovieAction;
 }
 
 class FormWidgetIface;
@@ -58,7 +59,6 @@ Q_OBJECT
         // Zoom mode ( last 4 are internally used only! )
         enum ZoomMode { ZoomFixed = 0, ZoomFitWidth = 1, ZoomFitPage = 2, ZoomFitText,
                         ZoomIn, ZoomOut, ZoomRefreshCurrent };
-        enum MouseMode { MouseNormal, MouseZoom, MouseSelect, MouseImageSelect, MouseTextSelect, MouseTableSelect };
 
         enum ClearMode { ClearAllSelection, ClearOnlyDividers };
 
@@ -126,9 +126,7 @@ Q_OBJECT
         void copyTextSelection() const;
         void selectAll();
 
-        void setAnnotationWindow( Okular::Annotation *annotation );
-
-        void removeAnnotationWindow( Okular::Annotation *annotation );
+        void openAnnotationWindow( Okular::Annotation *annotation, int pageNumber );
 
     signals:
         void urlDropped( const KUrl& );
@@ -246,6 +244,8 @@ Q_OBJECT
         void slotStopSpeaks();
         void slotAction( Okular::Action *action );
         void externalKeyPressEvent( QKeyEvent *e );
+        void slotAnnotationWindowDestroyed( QObject *window );
+        void slotProcessMovieAction( const Okular::MovieAction *action );
 };
 
 #endif
