@@ -219,6 +219,8 @@ PresentationWidget::PresentationWidget( QWidget * parent, Okular::Document * doc
     m_nextPageTimer->setSingleShot( true );
     connect( m_nextPageTimer, SIGNAL(timeout()), this, SLOT(slotNextPage()) ); 
 
+    connect( m_document, SIGNAL(processMovieAction(const Okular::MovieAction*)), this, SLOT(slotProcessMovieAction(const Okular::MovieAction*)) );
+
     // handle cursor appearance as specified in configuration
     if ( Okular::Settings::slidesCursor() == Okular::Settings::EnumSlidesCursor::HiddenDelay )
     {
@@ -356,7 +358,8 @@ void PresentationWidget::notifyPageChanged( int pageNumber, int changedFlags )
 
 bool PresentationWidget::canUnloadPixmap( int pageNumber ) const
 {
-    if ( Okular::Settings::memoryLevel() != Okular::Settings::EnumMemoryLevel::Aggressive )
+    if ( Okular::Settings::memoryLevel() == Okular::Settings::EnumMemoryLevel::Low ||
+         Okular::Settings::memoryLevel() == Okular::Settings::EnumMemoryLevel::Normal )
     {
         // can unload all pixmaps except for the currently visible one
         return pageNumber != m_frameIndex;
@@ -488,6 +491,17 @@ void PresentationWidget::mousePressEvent( QMouseEvent * e )
         if ( ( m_pressedLink = getLink( e->x(), e->y() ) ) )
             return;
 
+        const Okular::Annotation *annotation = getAnnotation( e->x(), e->y() );
+        if ( annotation && ( annotation->subType() == Okular::Annotation::AMovie ) )
+        {
+            const Okular::MovieAnnotation *movieAnnotation = static_cast<const Okular::MovieAnnotation*>( annotation );
+
+            VideoWidget *vw = m_frames[ m_frameIndex ]->videoWidgets.value( movieAnnotation->movie() );
+            vw->show();
+            vw->play();
+            return;
+        }
+
         // handle clicking on top-right overlay
         if ( !( Okular::Settings::slidesCursor() == Okular::Settings::EnumSlidesCursor::Hidden ) &&
              m_overlayGeometry.contains( e->pos() ) )
@@ -678,7 +692,7 @@ void PresentationWidget::leaveEvent( QEvent * e )
 }
 // </widget events>
 
-const Okular::Action * PresentationWidget::getLink( int x, int y, QRect * geometry ) const
+const void * PresentationWidget::getObjectRect( Okular::ObjectRect::ObjectType type, int x, int y, QRect * geometry ) const
 {
     // no links on invalid pages
     if ( geometry && !geometry->isNull() )
@@ -701,7 +715,7 @@ const Okular::Action * PresentationWidget::getLink( int x, int y, QRect * geomet
 
     // check if 1) there is an object and 2) it's a link
     const QRect d = QApplication::desktop()->screenGeometry( m_screen );
-    const Okular::ObjectRect * object = page->objectRect( Okular::ObjectRect::Action, nx, ny, d.width(), d.height() );
+    const Okular::ObjectRect * object = page->objectRect( type, nx, ny, d.width(), d.height() );
     if ( !object )
         return 0;
 
@@ -713,18 +727,32 @@ const Okular::Action * PresentationWidget::getLink( int x, int y, QRect * geomet
     }
 
     // return the link pointer
-    return (Okular::Action *)object->object();
+    return object->object();
+}
+
+const Okular::Action * PresentationWidget::getLink( int x, int y, QRect * geometry ) const
+{
+    return reinterpret_cast<const Okular::Action*>( getObjectRect( Okular::ObjectRect::Action, x, y, geometry ) );
+}
+
+const Okular::Annotation * PresentationWidget::getAnnotation( int x, int y, QRect * geometry ) const
+{
+    return reinterpret_cast<const Okular::Annotation*>( getObjectRect( Okular::ObjectRect::OAnnotation, x, y, geometry ) );
 }
 
 void PresentationWidget::testCursorOnLink( int x, int y )
 {
     const Okular::Action * link = getLink( x, y, 0 );
+    const Okular::Annotation *annotation = getAnnotation( x, y, 0 );
+
+    const bool needsHandCursor = ( ( link != 0 ) ||
+                                 ( ( annotation != 0 ) && ( annotation->subType() == Okular::Annotation::AMovie ) ) );
 
     // only react on changes (in/out from a link)
-    if ( (link && !m_handCursor) || (!link && m_handCursor) )
+    if ( ( needsHandCursor && !m_handCursor ) || ( !needsHandCursor && m_handCursor ) )
     {
         // change cursor shape
-        m_handCursor = link != 0;
+        m_handCursor = needsHandCursor;
         setCursor( QCursor( m_handCursor ? Qt::PointingHandCursor : Qt::ArrowCursor ) );
     }
 }
@@ -791,6 +819,10 @@ void PresentationWidget::changePage( int newPage )
         if ( m_document->page( m_frameIndex)->pageAction( Okular::Page::Opening ) )
             m_document->processAction( m_document->page( m_frameIndex )->pageAction( Okular::Page::Opening ) );
 
+        Q_FOREACH ( VideoWidget *vw, m_frames[ m_frameIndex ]->videoWidgets )
+        {
+            vw->pageEntered();
+        }
     }
 
     if ( oldIndex != m_frameIndex )
@@ -803,13 +835,17 @@ void PresentationWidget::changePage( int newPage )
                 vw->hide();
             }
         }
-
-        Q_FOREACH ( VideoWidget *vw, m_frames[ m_frameIndex ]->videoWidgets )
+        else
         {
-            vw->show();
-            vw->raise();
-        }
+            // we have just opened the presentation view
+            if ( m_document->page( m_frameIndex )->pageAction( Okular::Page::Opening ) )
+                m_document->processAction( m_document->page( m_frameIndex )->pageAction( Okular::Page::Opening ) );
 
+            Q_FOREACH ( VideoWidget *vw, m_frames[ m_frameIndex ]->videoWidgets )
+            {
+                vw->pageEntered();
+            }
+        }
     }
 }
 
@@ -1162,6 +1198,19 @@ void PresentationWidget::requestPixmaps()
             if ( !prevFrame->page->hasPixmap( PRESENTATION_ID, pixW, pixH ) )
                 requests.push_back( new Okular::PixmapRequest( PRESENTATION_ID, m_frameIndex - 1, pixW, pixH, PRESENTATION_PRELOAD_PRIO, true ) );
         }
+
+        // If greedy, preload everything
+        if (Okular::Settings::memoryLevel() == Okular::Settings::EnumMemoryLevel::Greedy)
+        {
+            for(int i = 0; i < (int)m_document->pages(); ++i)
+            {
+                PresentationFrame *loopFrame = m_frames[ i ];
+                pixW = loopFrame->geometry.width();
+                pixH = loopFrame->geometry.height();
+                if ( !loopFrame->page->hasPixmap( PRESENTATION_ID, pixW, pixH ))
+                    requests.push_back( new Okular::PixmapRequest( PRESENTATION_ID, i, pixW, pixH, PRESENTATION_PRELOAD_PRIO, true ) );
+            }
+        }
     }
     m_document->requestPixmaps( requests );
 }
@@ -1967,5 +2016,38 @@ void PresentationWidget::initTransition( const Okular::PageTransition *transitio
     m_transitionTimer->start( 0 );
 }
 
+void PresentationWidget::slotProcessMovieAction( const Okular::MovieAction *action )
+{
+    const Okular::MovieAnnotation *movieAnnotation = action->annotation();
+    if ( !movieAnnotation )
+        return;
+
+    Okular::Movie *movie = movieAnnotation->movie();
+    if ( !movie )
+        return;
+
+    VideoWidget *vw = m_frames[ m_frameIndex ]->videoWidgets.value( movieAnnotation->movie() );
+    if ( !vw )
+        return;
+
+    vw->show();
+
+    switch ( action->operation() )
+    {
+        case Okular::MovieAction::Play:
+            vw->stop();
+            vw->play();
+            break;
+        case Okular::MovieAction::Stop:
+            vw->stop();
+            break;
+        case Okular::MovieAction::Pause:
+            vw->pause();
+            break;
+        case Okular::MovieAction::Resume:
+            vw->play();
+            break;
+    };
+}
 
 #include "presentationwidget.moc"
