@@ -14,6 +14,7 @@
 #include <qpixmap.h>
 #include <qstringlist.h>
 #include <qwidget.h>
+#include "core/area.h"
 #include "core/observer.h"
 #include "core/pagetransition.h"
 
@@ -30,6 +31,7 @@ namespace Okular {
 class Action;
 class Annotation;
 class Document;
+class MovieAction;
 class Page;
 }
 
@@ -68,7 +70,9 @@ class PresentationWidget : public QWidget, public Okular::DocumentObserver
         void leaveEvent( QEvent * e );
 
     private:
+        const void * getObjectRect( Okular::ObjectRect::ObjectType type, int x, int y, QRect * geometry = 0 ) const;
         const Okular::Action * getLink( int x, int y, QRect * geometry = 0 ) const;
+        const Okular::Annotation * getAnnotation( int x, int y, QRect * geometry = 0 ) const;
         void testCursorOnLink( int x, int y );
         void overlayClick( const QPoint & position );
         void changePage( int newPage );
@@ -144,6 +148,7 @@ class PresentationWidget : public QWidget, public Okular::DocumentObserver
         void screenResized( int );
         void chooseScreen( QAction * );
         void toggleBlackScreenMode( bool );
+        void slotProcessMovieAction( const Okular::MovieAction *action );
 };
 
 #endif
